@@ -101,6 +101,7 @@ GMemChunk *sendbuffers = NULL, *recvbuffers = NULL;
 DEFINE_LOCK(request_id_counter);
 GIOP_unsigned_long request_id_counter;
 
+#if 0
 inline
 void giop_message_buffer_append_iovec(GIOPMessageBuffer *msgbuf,
 				      const struct iovec *iovec)
@@ -108,6 +109,9 @@ void giop_message_buffer_append_iovec(GIOPMessageBuffer *msgbuf,
   /* g_print("Appending iovec %d bytes @ %p\n", iovec->iov_len, iovec->iov_base); */
   g_array_append_val(msgbuf->iovecs, *iovec);
 }
+#else
+#define giop_message_buffer_append_iovec(msgbuf, iovec) g_array_append_val((msgbuf)->iovecs, *(iovec))
+#endif
 
 void
 giop_message_buffer_init(void)
@@ -161,7 +165,7 @@ giop_send_buffer_new(void)
   return msgbuf;
 }
 
-void
+gint
 giop_send_buffer_write(GIOPSendBuffer *send_buffer)
 {
   gulong nvecs;
@@ -171,7 +175,7 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
   GIOPConnection *cnx;
 
   cnx = GIOP_MESSAGE_BUFFER(send_buffer)->connection;
-  g_return_if_fail(cnx->is_valid);
+  g_return_val_if_fail(cnx->is_valid, -1);
 
   fd = GIOP_CONNECTION_GET_FD(cnx);
   nvecs = GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->len;
@@ -193,7 +197,7 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
   for(sum = res; sum < (GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size + sizeof(GIOPMessageHeader)); ) {
     if(res < 0) {
       giop_main_handle_connection_exception(cnx);
-      return;
+      return -1;
     }
 
     for(i = t = 0; t < sum; i++)
@@ -206,6 +210,8 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
 
     res = writev(fd, curvec, nvecs);
   }
+
+  return 0;
 }
 
 static GIOPSendBuffer *
@@ -260,9 +266,38 @@ giop_send_reply_buffer_use(GIOPConnection *connection,
 
   GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_REPLY;
 
-  ENCODER_CALL(IOP_ServiceContextList, service_context);
-  APIA(&request_id, sizeof(request_id));
-  APIA(&reply_status, sizeof(reply_status));
+  giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
+				   sizeof(GIOP_unsigned_long));
+  if(!service_context) {
+    static const GIOP_unsigned_long sc_zero_int = 0;
+    AP(&sc_zero_int, sizeof(service_context->_length));
+  } else {
+    int i, n;
+    n = service_context->_length;
+    AP(&service_context->_length, sizeof(service_context->_length));
+    for(i = 0; i < n; i++) {
+      int j, o;
+      CORBA_sequence_octet *seqo;
+
+      giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
+				       sizeof(GIOP_unsigned_long));
+      AP(&service_context->_buffer[i].context_id,
+	 sizeof(service_context->_buffer[i].context_id));
+      seqo = &service_context->_buffer[i].context_data;
+      o = seqo->_length;
+      AP(&seqo->_length, sizeof(GIOP_unsigned_long));
+      for(j = 0; j < o; j++)
+	AP(seqo->_buffer, seqo->_length);
+    }
+  }
+  send_buffer->message.u.reply.request_id = request_id;
+  send_buffer->message.u.reply.reply_status = reply_status;
+  giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
+				   sizeof(GIOP_unsigned_long));
+  AP(&send_buffer->message.u.reply.request_id,
+     sizeof(GIOP_unsigned_long));
+  AP(&send_buffer->message.u.reply.reply_status,
+     sizeof(GIOP_unsigned_long));
 
   return send_buffer;
 }
@@ -318,12 +353,40 @@ giop_send_request_buffer_use(GIOPConnection *connection,
 
   GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_REQUEST;
 
-  ENCODER_CALL(IOP_ServiceContextList, service_context);
-  APIA(&request_id, sizeof(request_id));
+  giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
+				   sizeof(GIOP_unsigned_long));
+  if(!service_context) {
+    static const GIOP_unsigned_long sc_zero_int = 0;
+    AP(&sc_zero_int, sizeof(GIOP_unsigned_long));
+  } else {
+    int i, n;
+    n = service_context->_length;
+    AP(&service_context->_length, sizeof(service_context->_length));
+    for(i = 0; i < n; i++) {
+      int j, o;
+      CORBA_sequence_octet *seqo;
 
-  response_expected = response_expected?1:0;
-  API(&response_expected, 1);
+      giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
+				       sizeof(GIOP_unsigned_long));
+      AP(&service_context->_buffer[i].context_id,
+	 sizeof(service_context->_buffer[i].context_id));
+      seqo = &service_context->_buffer[i].context_data;
+      o = seqo->_length;
+      AP(&seqo->_length, sizeof(GIOP_unsigned_long));
+      for(j = 0; j < o; j++)
+	AP(seqo->_buffer, seqo->_length);
+    }
+  }
+  send_buffer->message.u.request.request_id = request_id;
+  send_buffer->message.u.request.response_expected = response_expected;
+  giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
+				   sizeof(GIOP_unsigned_long));
+  AP(&send_buffer->message.u.request.request_id,
+     sizeof(GIOP_unsigned_long));
+  AP(&send_buffer->message.u.request.response_expected,
+     sizeof(GIOP_boolean));
 #if 0
+  API(&response_expected, 1);
   AP((gpointer)giop_scratch_space, 3);
 #endif
 
