@@ -124,7 +124,7 @@ static void ORBit_rc_load(const char *rcfile, ORBit_orb_options *options)
 		char buf[1024];
 
 		if(fgets(buf, 1024, fp)!=NULL) {
-			char *bptr, *tmp, *key, *data;
+			guchar *bptr, *tmp, *key, *data;
 			size_t start, len;
 
 			if(buf[0]=='#')
